@@ -1056,25 +1056,21 @@ ORBit_value_equivalent (gpointer *a, gpointer *b,
 	}
 
 	case CORBA_tk_union: {
-		gint     union_align = tc->c_align;
-		gint     discrim_align = MAX (tc->discriminator->c_align, tc->c_align);
-		size_t   union_size = ORBit_gather_alloc_info (tc);
-		gpointer a_orig, b_orig;
+		CORBA_TypeCode utc_a, utc_b;
+		gint           union_align = tc->c_align;
+		gint           discrim_align = MAX (tc->discriminator->c_align, tc->c_align);
+		size_t         union_size = ORBit_gather_alloc_info (tc);
+		gpointer       a_orig, b_orig;
 
-		CORBA_TypeCode utc_a = ORBit_get_union_tag (
-			tc, (gconstpointer *)a, FALSE);
-		CORBA_TypeCode utc_b = ORBit_get_union_tag (
-			tc, (gconstpointer *)b, FALSE);
+		a_orig = *a = ALIGN_ADDRESS (*a, discrim_align);
+		b_orig = *b = ALIGN_ADDRESS (*b, discrim_align);
 
-		a_orig = *a;
-		b_orig = *b;
+		utc_a = ORBit_get_union_tag (tc, (gconstpointer *)a, FALSE);
+		utc_b = ORBit_get_union_tag (tc, (gconstpointer *)b, FALSE);
 
 		if (!CORBA_TypeCode_equal (utc_a, utc_b, ev))
 			return FALSE;
 
-		*a = ALIGN_ADDRESS (*a, discrim_align);
-		*b = ALIGN_ADDRESS (*b, discrim_align);
-
 		if (!ORBit_value_equivalent (a, b, tc->discriminator, ev))
 			return FALSE;
 
@@ -1165,8 +1161,10 @@ ORBit_sequence_alloc (CORBA_TypeCode      sequence_tc,
 	
 	seq = ORBit_alloc_by_tc (sequence_tc);
 	seq->_buffer = ORBit_small_allocbuf (tc, length);
-	seq->_length = length;
+	seq->_length  = length;
 	seq->_maximum = length;
+	
+	CORBA_sequence_set_release (seq, CORBA_TRUE);
 
 	g_assert (ORBit_alloc_get_tcval (seq) == sequence_tc);
 
@@ -1241,11 +1239,12 @@ ORBit_sequence_append (gpointer      sequence,
 	g_return_if_fail (tc->kind == CORBA_tk_sequence);
 
 	if (seq->_length == seq->_maximum) {
-		/* NB. extend maximum, not length */
+		guint new_len = MAX (2, (seq->_maximum * 2));
+
 		seq->_buffer = ORBit_realloc_tcval
 			(seq->_buffer, subtc,
-			 seq->_maximum, seq->_maximum * 2);
-		seq->_maximum *= 2;
+			 seq->_maximum, new_len );
+		seq->_maximum = new_len;
 	}
 
 	element_size = ORBit_gather_alloc_info (subtc);
@@ -1261,6 +1260,24 @@ void
 ORBit_sequence_concat (gpointer      sequence,
 		       gconstpointer append)
 {
-	/* FIXME: implement me */
-	g_warning ("Implement me");
+	gint  i;	
+	guint element_size;
+	guchar *src;
+	CORBA_TypeCode tc, subtc;
+  	CORBA_sequence_CORBA_octet *seq = (CORBA_sequence_CORBA_octet *)append;
+
+	g_return_if_fail (seq != NULL);
+	g_return_if_fail (seq->_length <= seq->_maximum);
+
+	tc = ORBit_alloc_get_tcval (sequence);
+	SKIP_ALIAS (tc);
+	subtc = tc->subtypes [0];
+	g_return_if_fail (tc->kind == CORBA_tk_sequence);
+
+	element_size = ORBit_gather_alloc_info (subtc);
+
+	src = seq->_buffer;
+
+	for (i = 0; i < seq->_length; ++i, src += element_size) 
+		ORBit_sequence_append (sequence, (gpointer)src);
 }
